@@ -51,11 +51,11 @@ would probably load this using the Bio.AlignIO.parse() function:
     >>> for aln in alignments:
     ...     print(aln)
     ...
-    SingleLetterAlphabet() alignment with 3 rows and 240 columns
+    Alignment with 3 rows and 240 columns
     --------------------------------------------...--- a.fa
     TTTAAACATCCCTCGGCCCGTCGCCCTTTTATAATAGCAGTACG...CTG b.fa/5416-5968
     TTTAAACACCTTTTTGGATG--GCCCAGTTCGTTCAGTTGTG-G...CTT c.fa/9475-10076
-    SingleLetterAlphabet() alignment with 2 rows and 46 columns
+    Alignment with 2 rows and 46 columns
     -----------------------GGGCGAACGTATAAACCATTCTG b.fa/5968-6015
     TTCGGTACCCTCCATGACCCACGAAATGAGGGCCCAGGGTATGCTT c.fa/9428-9476
 
@@ -76,12 +76,12 @@ the annotation attribute of each record::
       start: 9475, end: 10076, strand: -1
 
 """
-
-
 import re
+
+from Bio.Align import MultipleSeqAlignment
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
-from Bio.Align import MultipleSeqAlignment
+
 from .Interfaces import AlignmentIterator
 from .Interfaces import SequentialAlignmentWriter
 
@@ -107,7 +107,7 @@ class MauveWriter(SequentialAlignmentWriter):
     """Mauve/XMFA alignment writer."""
 
     def __init__(self, *args, **kwargs):
-        """Initialize."""
+        """Initialize the class."""
         super().__init__(*args, **kwargs)
         self._wrote_header = False
         self._wrote_first = False
@@ -139,7 +139,7 @@ class MauveWriter(SequentialAlignmentWriter):
             # Sequence1Entry	1
             # Sequence1Format	FastA
             for i in range(1, count + 1):
-                self.handle.write("#Sequence%sEntry\t%s\n" % (i, i))
+                self.handle.write(f"#Sequence{i}Entry\t{i}\n")
 
         for idx, record in enumerate(alignment):
             self._write_record(record, record_idx=idx)
@@ -159,14 +159,8 @@ class MauveWriter(SequentialAlignmentWriter):
         # We remove the "/{start}-{end}" before writing, as it cannot be part
         # of the produced XMFA file.
         if "start" in record.annotations and "end" in record.annotations:
-            suffix0 = "/%s-%s" % (
-                str(record.annotations["start"]),
-                str(record.annotations["end"]),
-            )
-            suffix1 = "/%s-%s" % (
-                str(record.annotations["start"] + 1),
-                str(record.annotations["end"]),
-            )
+            suffix0 = f"/{record.annotations['start']}-{record.annotations['end']}"
+            suffix1 = f"/{record.annotations['start'] + 1}-{record.annotations['end']}"
             if seq_name[-len(suffix0) :] == suffix0:
                 seq_name = seq_name[: -len(suffix0)]
             if seq_name[-len(suffix1) :] == suffix1:
@@ -216,14 +210,14 @@ class MauveWriter(SequentialAlignmentWriter):
                 id_line = id_line.replace("\n", " ").replace("\r", " ")
                 self.handle.write(id_line + "\n\n")
             # Alignments lacking a start/stop/strand were generated by
-            # BioPython on load, and shouldn't exist according to XMFA
+            # Biopython on load, and shouldn't exist according to XMFA
         else:
             # In other blocks, we only write sequences if they exist in a given
             # alignment.
             id_line = id_line.replace("\n", " ").replace("\r", " ")
             self.handle.write(id_line + "\n")
             for i in range(0, len(record.seq), 80):
-                self.handle.write("%s\n" % str(record.seq[i : i + 80]))
+                self.handle.write(f"{record.seq[i:i + 80]}\n")
 
 
 class MauveIterator(AlignmentIterator):
@@ -335,7 +329,7 @@ class MauveIterator(AlignmentIterator):
                     else:
                         corrected_id = seq_regions[id]["name"]
 
-                record = SeqRecord(Seq(seq, self.alphabet), id=corrected_id, name=id)
+                record = SeqRecord(Seq(seq), id=corrected_id, name=id)
 
                 record.annotations["start"] = seq_regions[id]["start"]
                 record.annotations["end"] = seq_regions[id]["end"]
@@ -344,6 +338,6 @@ class MauveIterator(AlignmentIterator):
                 )
 
                 records.append(record)
-            return MultipleSeqAlignment(records, self.alphabet)
+            return MultipleSeqAlignment(records)
         else:
             raise StopIteration
