@@ -6,7 +6,7 @@
 # choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
 # Please see the LICENSE file that should have been included as part of this
 # package.
-"""Definitions for interacting with BLAST related applications.
+"""Definitions for interacting with BLAST related applications (OBSOLETE).
 
 Wrappers for the new NCBI BLAST+ tools (written in C++):
 
@@ -27,6 +27,9 @@ For further details, see:
 Camacho et al. BLAST+: architecture and applications
 BMC Bioinformatics 2009, 10:421
 https://doi.org/10.1186/1471-2105-10-421
+
+We have decided to remove this module in future, and instead recommend
+building your command and invoking it via the subprocess module directly.
 """
 
 from Bio.Application import _Option, AbstractCommandline, _Switch
@@ -123,7 +126,7 @@ class _NcbibaseblastCommandline(AbstractCommandline):
             if self._get_parameter(a):
                 for b in incompatibles[a]:
                     if self._get_parameter(b):
-                        raise ValueError("Options %s and %s are incompatible." % (a, b))
+                        raise ValueError(f"Options {a} and {b} are incompatible.")
 
 
 class _NcbiblastCommandline(_NcbibaseblastCommandline):
@@ -1452,6 +1455,14 @@ class NcbimakeblastdbCommandline(AbstractCommandline):
                 equate=False,
             ),
             # makeblastdb specific options
+            _Option(
+                ["-blastdb_version", "blastdb_version"],
+                "Version of BLAST database to be created. "
+                "Tip: use BLAST database version 4 on 32 bit CPU. "
+                "Default = 5",
+                equate=False,
+                checker_function=lambda x: x == 4 or x == 5,
+            ),
             _Option(
                 ["-dbtype", "dbtype"],
                 "Molecule type of target db ('nucl' or 'prot').",
@@ -1545,8 +1556,8 @@ class NcbimakeblastdbCommandline(AbstractCommandline):
         ]
         AbstractCommandline.__init__(self, cmd, **kwargs)
 
-    def _input_type_checker(command, x):
-        return x in ("asn1_bin", "asn1_txt", "blastdb", "fasta")
+    def _input_type_checker(self, command):
+        return command in ("asn1_bin", "asn1_txt", "blastdb", "fasta")
 
     def _validate(self):
         incompatibles = {
@@ -1556,13 +1567,13 @@ class NcbimakeblastdbCommandline(AbstractCommandline):
         }
 
         # Copied from _NcbibaseblastCommandline class above.
-        # Code repeated here for python2 and 3 comptaibility,
+        # Code repeated here for python2 and 3 compatibility,
         # because this is not a _NcbibaseblastCommandline subclass.
         for a in incompatibles:
             if self._get_parameter(a):
                 for b in incompatibles[a]:
                     if self._get_parameter(b):
-                        raise ValueError("Options %s and %s are incompatible." % (a, b))
+                        raise ValueError(f"Options {a} and {b} are incompatible.")
 
         if self.mask_id and not self.mask_data:
             raise ValueError("Option mask_id requires mask_data to be set.")
