@@ -82,7 +82,7 @@ BlatIO provides the following attribute-column mapping:
 +----------------+-------------------------+-----------------------------------+
 | Object         | Attribute               | Column Name, Value                |
 +================+=========================+===================================+
-| QueryResutl    | id                      | Q name, query sequence ID         |
+| QueryResult    | id                      | Q name, query sequence ID         |
 |                +-------------------------+-----------------------------------+
 |                | seq_len                 | Q size, query sequence full       |
 |                |                         | length                            |
@@ -182,7 +182,6 @@ HSP and HSPFragment documentation for more details on these properties.
 import re
 from math import log
 
-from Bio.Alphabet import generic_dna
 from Bio.SearchIO._index import SearchIndexer
 from Bio.SearchIO._model import QueryResult, Hit, HSP, HSPFragment
 
@@ -356,8 +355,8 @@ def _create_hsp(hid, qid, psl):
         qseqlist = psl.get("qseqs")
         qseq = "" if not qseqlist else qseqlist[idx]
         frag = HSPFragment(hid, qid, hit=hseq, query=qseq)
-        # set alphabet
-        frag.alphabet = generic_dna
+        # set molecule type
+        frag.molecule_type = "DNA"
         # set coordinates
         frag.query_start = qcoords[0]
         frag.query_end = qcoords[1]
